/*****************************************************************************/
/*                                                                           */
/*                                ICCONFIG.H                                 */
/*                                                                           */
/* (C) 1995-96  Ullrich von Bassewitz                                        */
/*              Wacholderweg 14                                              */
/*              D-70597 Stuttgart                                            */
/* EMail:       uz@ibb.schwaben.com                                          */
/*                                                                           */
/*****************************************************************************/



// $Id$
//
// $Log$
//
//



// Please notice: All structs defined in this module must be char aligned
// internally. See isteccfg.txt (author unknown).



#ifndef _ICCONFIG_H
#define _ICCONFIG_H



#include <string.h>

#include "icver.h"
#include "istecmsg.h"



/*****************************************************************************/
/*                                 Constants                                 */
/*****************************************************************************/



// Possible values for IstecBaseConfig.Connection
const unsigned coMehrgeraete            = 0;
const unsigned coAnlagen                = 1;
const unsigned coMax                    = 1;

// Possible values for IstecBaseConfig.Protocol
const unsigned pr1TR6                   = 0;
const unsigned prDSS1                   = 1;
const unsigned prMax                    = 1;

// Possible values for IstecBaseConfig.DialCaps
const unsigned dcKeine                  = 0;
const unsigned dcInland                 = 1;
const unsigned dcOrt                    = 2;
const unsigned dcHalbamt                = 3;
const unsigned dcNichtamt               = 4;
const unsigned dcMax                    = 4;

// Possible values for IstecBaseConfig.Service
const unsigned svFernsprechen           = 0;
const unsigned svFaxG3                  = 1;
const unsigned svDatenModem             = 2;
const unsigned svDatexJModem            = 3;
const unsigned svAnrufbeantworter       = 4;
const unsigned svKombi                  = 5;

// Flags for the internal and external knock bitsets
const unsigned knInt21                  = 0x0001;
const unsigned knInt22                  = 0x0002;
const unsigned knInt23                  = 0x0004;
const unsigned knInt24                  = 0x0008;
const unsigned knInt25                  = 0x0010;
const unsigned knInt26                  = 0x0020;
const unsigned knInt27                  = 0x0040;
const unsigned knInt28                  = 0x0080;
const unsigned knMSN0                   = 0x0001;
const unsigned knMSN1                   = 0x0002;
const unsigned knMSN2                   = 0x0004;
const unsigned knMSN3                   = 0x0008;
const unsigned knMSN4                   = 0x0010;
const unsigned knMSN5                   = 0x0020;
const unsigned knMSN6                   = 0x0040;
const unsigned knMSN7                   = 0x0080;
const unsigned knMSN8                   = 0x0100;
const unsigned knMSN9                   = 0x0200;
const unsigned knTFE1                   = 0x0400;
const unsigned knTFE2                   = 0x0800;
const unsigned knTFE3                   = 0x1000;
const unsigned knTFE4                   = 0x2000;

// Count of MSN's
const unsigned bcMSNCount               = 10;

// Maximum count of devices
const unsigned IstecDevCount            = 64;

// Raw sizes of the transmission data buffers. The actual space occupied from
// the date maybe less, depending on the istec firmware version.
const unsigned BaseConfigSize           = 106;
const unsigned DevConfigSize            = 22;
const unsigned ChargeSize               = 128;



/*****************************************************************************/
/*                           class IstecBaseConfig                           */
/*****************************************************************************/



// Basic ISTEC configuration struct
class IstecBaseConfig {

public:
    unsigned char       Connection;
    unsigned char       AB_InterfaceCount;
    unsigned char       Protocol;
    unsigned char       VersionHigh;
    unsigned char       VersionLow;
    unsigned char       TFEAssignment;

    unsigned char       MSN [bcMSNCount] [5];   // MSN in BCD

    unsigned char       Music;                  // 0 == off, 1 == on
    unsigned char       IntS0;
    unsigned char       ExtS0;

    unsigned char       QueryLoc1;
    unsigned char       QueryLoc2;
    unsigned char       Number1 [11];           // Number 1, pascal style
    unsigned char       Number2 [11];           // Number 2, pascal style

    unsigned char       MSNGroups [10];

//  ----------------------------------------    // 1.93 and up
    unsigned char       MusicPort;
    unsigned            CountryCode;
    unsigned char       TFELoc [4];             // TFE Location (bitmap)



    unsigned char* Pack (unsigned char* Buf) const;
    // Pack the data of struct IstecBaseConfig into an array ready for
    // transmission. The function returns Buf.

    IstecBaseConfig& Unpack (const unsigned char* Buf);
    IstecBaseConfig& Unpack (const IstecMsg& Msg);
    // Unpack an array of char that contains data for an IstecBaseConfig struct.
    // The function returns *this and corrects invalid raw values.

    friend int operator == (const IstecBaseConfig&, const IstecBaseConfig&);
    friend int operator != (const IstecBaseConfig&, const IstecBaseConfig&);
    // Compare two structs

};



/*****************************************************************************/
/*                                class DevConfig                            */
/*****************************************************************************/



// Configuration of the devices device
class DevConfig {

public:
    unsigned char       DevNum;
    unsigned char       DialCaps;
    unsigned char       Service;
    unsigned char       Reroute;
    unsigned char       ChargePulse;
    unsigned char       PIN [2];
    unsigned char       ExtNum [10];

//  ---------------------------------   // 1.93 and up
    unsigned char       TerminalMode;

//  ---------------------------------   // 1.94 and up
    unsigned char       InternalKnock;
    unsigned            ExternalKnock;


    void ClearExtNum ();
    // Clear the external number

    unsigned char* Pack (unsigned char* Buf,
                         double aFirmwareVersion = FirmwareVersion) const;
    // Pack the data of struct DevConfig into an array ready for transmission.
    // The function returns Buf.

    DevConfig& Unpack (const unsigned char* Buf,
                       double aFirmware = FirmwareVersion);
    DevConfig& Unpack (const IstecMsg& Msg, double aFirmware = FirmwareVersion);
    // Unpack an array of char that contains data for a DevConfig struct.
    // The function returns this.

    friend int operator == (const DevConfig&, const DevConfig&);
    friend int operator != (const DevConfig&, const DevConfig&);
    // Compare two structs

    int GetIntKnock (unsigned Bit) const;
    // Return true if the internal knock bit is set

    int GetExtKnock (unsigned Bit) const;
    // Return true if the external knock bit is set

    void SetIntKnock (unsigned Bit);
    // Set the interal knock to true

    void SetExtKnock (unsigned Bit);
    // Set the external knock to true

    void ClrIntKnock (unsigned Bit);
    // Set the interal knock to false

    void ClrExtKnock (unsigned Bit);
    // Set the external knock to false

};

inline int DevConfig::GetIntKnock (unsigned Bit) const
// Return true if the internal knock bit is set
{
    return (InternalKnock & Bit) != 0;
}



inline int DevConfig::GetExtKnock (unsigned Bit) const
// Return true if the external knock bit is set
{
    return (ExternalKnock & Bit) != 0;
}



inline void DevConfig::SetIntKnock (unsigned Bit)
// Set the interal knock to true
{
    InternalKnock |= Bit;
}



inline void DevConfig::SetExtKnock (unsigned Bit)
// Set the external knock to true
{
    ExternalKnock |= Bit;
}



inline void DevConfig::ClrIntKnock (unsigned Bit)
// Set the interal knock to false
{
    InternalKnock &= ~Bit;
}



inline void DevConfig::ClrExtKnock (unsigned Bit)
// Set the external knock to false
{
    InternalKnock &= ~Bit;
}



typedef DevConfig       IstecDevConfig [IstecDevCount];



/*****************************************************************************/
/*                             class IstecCharges                            */
/*****************************************************************************/



class IstecCharges {

    unsigned Charges [IstecDevCount];

public:
    IstecCharges ();
    // Constructor - clears the charges on startup

    void Clear ();
    // Clear the charges

    unsigned& operator [] (unsigned Device);
    // Return a reference to the charges of a specific device

    const unsigned& operator [] (unsigned Device) const;
    // Return a reference to the charges of a specific device

    unsigned char* Pack (unsigned char* Buf) const;
    // Pack the data of IstecCharges into an array ready for transmission.
    // The function returns Buf.

    IstecCharges& Unpack (const unsigned char* Buf);
    // Unpack an array of char that contains data for an IstecCharges struct.
    // The function returns this.

    friend int operator == (const IstecCharges&, const IstecCharges&);
    friend int operator != (const IstecCharges&, const IstecCharges&);
    // Compare two structs

};



inline IstecCharges::IstecCharges ()
// Constructor - clears the charges on startup
{
    Clear ();
}



/*****************************************************************************/
/*                                   Code                                    */
/*****************************************************************************/



unsigned char* ToBCD (const char* S, unsigned char* T, unsigned TSize);
// Convert the number string in S to a BCD representation in T, filling unused
// digits with 'F'. The function returns T.

char* FromBCD (char* S, const unsigned char* T, unsigned TSize);
// Convert the BCD string in T into an ASCII representation in S. Conversion
// stops if an invalid BCD char or TSize is reached. A trailing zero is added
// to S. It is assumed that S is big anough to hold the resulting string (S
// must be TSize*2+1 chars in size).
// The function returns S.

unsigned char* ToPascal (const char* S, unsigned char* T, unsigned TSize);
// Convert the C style string in S to the pascal string in T and return T.

char* FromPascal (char* S, const unsigned char* T, unsigned SSize);
// Convert the pascal style string in T into a C like string in S and return S.



// End of ICCONFIG.H

#endif

